package master

import (
	"fmt"

	"cant/can/canbus"
	"cant/can/flowroutines/canflow"
	"cant/can/flowroutines/filter"
	"cant/can/flowroutines/isobusflow"
	"cant/cmd/run"
	"cant/util/flow"
	"cant/util/globals"
	"cant/webserver"

	// import master routes
	_ "cant/webserver/routing/master"
	//"github.com/pkg/profile"
	"github.com/spf13/cobra"
)

var mode string

// init adds the seed command as subcommand to the Root
func init() {
	run.Run.AddCommand(master)
	master.PersistentFlags().StringVarP(&mode, "mode", "m", "filter", "operation mode, valid operations: passthru, identify, filter")
}

// run command
var master = &cobra.Command{
	Use:   "master",
	Short: "run the master component",
	Run: func(cobraCmd *cobra.Command, args []string) {
		// create flowgraph
		can0, err := canbus.New(globals.Config.Network.Interface0)
		if err != nil {
			fmt.Println(err)
			return
		}
		defer can0.Close()
		can1, err := canbus.New(globals.Config.Network.Interface1)
		if err != nil {
			fmt.Println(err)
			return
		}
		defer can1.Close()
		reader0 := canflow.CreateReader(can0)
		writer0 := canflow.CreateWriter(can0)
		reader1 := canflow.CreateReader(can1)
		writer1 := canflow.CreateWriter(can1)
		reader0.DefaultRoute(writer1)
		reader1.DefaultRoute(writer0)
		identifier0 := isobusflow.CreateIdentifier()
		identifier1 := isobusflow.CreateIdentifier()
		dataflow := flow.New()
		switch mode {
		case "passthru":
			dataflow.Connect(reader0, writer1)
			dataflow.Connect(reader1, writer0)
		case "identify":
			dataflow.Connect(reader0, identifier0)
			dataflow.Connect(identifier0, writer1)
			dataflow.Connect(reader1, identifier1)
			dataflow.Connect(identifier1, writer0)
		case "filter":
			filter0 := filter.CreateDynamicFilter()
			filter1 := filter.CreateDynamicFilter()
			dataflow.Connect(reader0, identifier0)
			dataflow.Connect(identifier0, filter0)
			dataflow.Connect(filter0, writer1)
			dataflow.Connect(filter0, writer0)
			dataflow.Connect(reader1, identifier1)
			dataflow.Connect(identifier1, filter1)
			dataflow.Connect(filter1, writer0)
			dataflow.Connect(filter1, writer1)
		}
		dataflow.Start()
		// run webserver
		webserver.Serve()
		//dataflow.Join()

	},
}
