var bootstrap = window.ReactBootstrap;
var reactRouter = window.ReactRouterDOM;

class App extends React.Component {

    render() {
        return (
            <reactRouter.HashRouter>
                <div id="page-content-wrapper">
                    <Navigation />
                    <div class="container-fluid">
                        <reactRouter.Route path="/pgn" component={PGN}/>
                        <reactRouter.Route path="/pgn/:pgn_id" component={PGNDetails}/>
                        <reactRouter.Route path="/spn" component={SPN}/>
                        <reactRouter.Route path="/spn/:spn_id" component={SPNDetails}/>
                        <reactRouter.Route path="/statistics" component={Statistics}/>
                    </div>
                </div>
            </reactRouter.HashRouter>
        )
    }
}
class Navigation extends React.Component {
    render() {
        return (
          <bootstrap.Navbar inverse collapseOnSelect fixedTop fluid>
            <bootstrap.Navbar.Header>
              <bootstrap.Navbar.Brand>
                <a href="#">CANT</a>
              </bootstrap.Navbar.Brand>
              <bootstrap.Navbar.Toggle />
            </bootstrap.Navbar.Header>
            <bootstrap.Navbar.Collapse>
              <bootstrap.Nav>
                <NavItem name="PGN" />
                <NavItem name="SPN" />
                <NavItem name="Statistics" />
              </bootstrap.Nav>
            </bootstrap.Navbar.Collapse>
          </bootstrap.Navbar>
        )
    }
}

class NavItem extends React.Component {
    render() {
        return (
            <li class="nav-item">
                <reactRouter.Link to={this.props.name.toLowerCase()} class="nav-link">
                    {this.props.name}
                </reactRouter.Link>
            </li>
        )
    }
}

class PageInfo extends React.Component {
    render() {
        return (
            <bootstrap.PanelGroup accordion>
                <bootstrap.Panel eventKey="1" header={this.props.header} bsStyle="info">
                    {this.props.children}
                </bootstrap.Panel>
            </bootstrap.PanelGroup>
        )
    }
}

class Statistics extends React.Component {
    constructor() {
        super();
        this.state = {
            stats: null,
            log: []
        }
    }

    componentDidMount() {
        this.statws = new WebSocket("ws://" + location.host + "/api/stats")
        this.statws.onmessage = function (e) {
            let data = JSON.parse(e.data)
            this.setState({stats: data})
        }.bind(this);
        this.logws = new WebSocket("ws://" + location.host + "/api/livelog")
        this.logws.onmessage = function (e) {
            let log = this.state.log
            if(log.length > 50) {
                log.shift()
            }
            log.push(JSON.parse(e.data))
            this.setState({log: log})
        }.bind(this);
    }

    componentWillUnmount() {
        this.statws.close()
        this.logws.close()
    }

    render() {
        if(this.state.stats == null) {
            return (
                <div id="statistics">
                </div>
            )
        }

        return (
            <div id="statistics">
                <PageInfo header="Statistics">
                    This is the Statistics page. You can view statistics regarding the CAN connection.
                </PageInfo>
                <h1>Summary</h1>
                <bootstrap.Table striped bordered condensed hover>
                    <thead>
                        <tr>
                            <th scope="col">Identified Frames</th>
                            <th scope="col">Unknown Frames</th>
                            <th scope="col">Blocked Frames</th>
                            <th scope="col">Manipulated Frames</th>
                            <th scope="col">TX Frames</th>
                            <th scope="col">TX Bytes</th>
                            <th scope="col">RX Frames</th>
                            <th scope="col">RX Bytes</th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr>
                            <td>{this.state.stats.identified}</td>
                            <td>{this.state.stats.unknown}</td>
                            <td>{this.state.stats.blocked}</td>
                            <td>{this.state.stats.manipulated}</td>
                            <td>{this.state.stats.sum_tx}</td>
                            <td>{this.state.stats.sum_tx * 16}</td>
                            <td>{this.state.stats.sum_rx}</td>
                            <td>{this.state.stats.sum_rx * 16}</td>
                        </tr>
                    </tbody>
                </bootstrap.Table>
                <h1>Interfaces</h1>
                <bootstrap.Table striped bordered condensed hover>
                    <thead>
                        <tr>
                            <th scope="col">Interface</th>
                            <th scope="col">TX Frames</th>
                            <th scope="col">TX Bytes</th>
                            <th scope="col">RX Frames</th>
                            <th scope="col">RX Bytes</th>
                        </tr>
                    </thead>
                    <tbody>
                        {Object.keys(this.state.stats.interfaces).map(iface => {
                            return (
                                <tr>
                                    <td>{iface}</td>
                                    <td>{this.state.stats.interfaces[iface].out_pkts}</td>
                                    <td>{this.state.stats.interfaces[iface].out_pkts * 16}</td>
                                    <td>{this.state.stats.interfaces[iface].in_pkts}</td>
                                    <td>{this.state.stats.interfaces[iface].in_pkts * 16}</td>
                                </tr>
                            )
                        })}
                    </tbody>
                </bootstrap.Table>
                <h1>Live Log</h1>
                <pre class="pre-scrollable">
                    <code>
                        {this.state.log.map( line => {
                            return (
                                <p>{line.timestamp + " - " + line.identifier + " - " + line.msg}</p>
                            )
                        })}
                    </code>
                </pre>
            </div>
        )
    }
}


class PGN extends React.Component {
    constructor() {
        super();
        this.state = {pgn: []};
    }

    componentDidMount() {
        fetch("/api/pgn")
                .then( (response) => {
                    return response.json() })   
                        .then( (json) => {
                                this.setState({pgn: json});
                        });
    }

    openDetails(id) {
        window.location = "/#/pgn/" + id;
    }

    render() {
        let data = [];
        for (var i = 0; i < this.state.pgn.length; i++) {
            data.push({
                data: [this.state.pgn[i].id, this.state.pgn[i].name],
                onClickRow: this.openDetails,
                paramsOnClick: this.state.pgn[i].id
            });
        };
        let targets = [
            {name: "ID", field: 0, apply: "toString", apply_arg: ""},
            {name: "Name", field: 1, apply: "", apply_arg: ""}
        ];
        return (
            <div id="pgn-panel">
                <PageInfo header="PGN">
                    This is the PGN panel. You can browse and search all known PGNs provided by ISO 11783-1. 
                    This database is read-only and has been fetched from the publicly available <a href="https://www.isobus.net/isobus/">ISO 11783-1 PGN online database</a> provided by <a href="https://lt.vdma.org/">VDMA Fachverband Landtechnik</a>.
                </PageInfo>
                <SearchableNavigatableTable targets={targets} header={["ID", "Name"]} data={data} />
            </div>
        )
    }
}

class SPN extends React.Component {
    constructor() {
        super();
        this.state = {spn: []}
    }

    componentDidMount() {
        fetch("/api/spn")
            .then( (response) => {
                return response.json() })   
                    .then( (json) => {
                            this.setState({spn: json});
                    });
    }

    openDetails(id) {
        window.location = "/#/spn/" + id;
    }

    render() {
        let data = [];
        for (var i = 0; i < this.state.spn.length; i++) {
            data.push({
                data: [this.state.spn[i].id, this.state.spn[i].name],
                onClickRow: this.openDetails,
                paramsOnClick: this.state.spn[i].id
            });
        };
        let targets = [
            {name: "ID", field: 0, apply: "toString", apply_arg: ""},
            {name: "Name", field: 1, apply: "", apply_arg: ""},
        ];
        return (
            <div id="spn-panel">
                <PageInfo header="SPN">
                    This is the SPN panel. You can browse and search all known SPNs provided by ISO 11783-1. 
                    This database is read-only and has been fetched from the publicly available <a href="https://www.isobus.net/isobus/">ISO 11783-1 SPN online database</a> provided by <a href="https://lt.vdma.org/">VDMA Fachverband Landtechnik</a>.
                </PageInfo>
                <SearchableNavigatableTable targets={targets} header={["ID", "Name"]} data={data} />
            </div>
        )
    }
}

class SPNDetails extends React.Component {

    constructor() {
        super();
        this.state = {
            spn: {id: 0, name: "", pgn: 0}
        }
    }

    componentDidMount() {
        fetch("/api/spn/" + this.props.match.params.spn_id)
            .then( (response) => {
                return response.json() })   
                    .then( (json) => {
                        this.setState({spn: json});
                    });
    }

    closeDetails() {
        window.location = "/#/spn";
    }

    render() {
        let pgn_url = "/#/pgn/" + this.state.spn.pgn;
        let attributes = [
            {name: "ID", value: this.state.spn.id.toString()},
            {name: "Name", value: this.state.spn.name},
            {name: "Associated pgn", value: <a target="_blank" href={pgn_url}>{this.state.spn.pgn}</a>},
        ];
        return (
            <bootstrap.Modal show={true} onHide={this.closeDetails}>
              <bootstrap.Modal.Header>
                <bootstrap.Modal.Title><strong>SPN {this.state.spn.id}</strong></bootstrap.Modal.Title>
              </bootstrap.Modal.Header>
              <bootstrap.Modal.Body>
            <bootstrap.Table striped bordered condensed hover>
                <thead>
                    <tr>
                        <th scope="col">Attribute</th>
                        <th scope="col">Value</th>
                    </tr>
                </thead>
                <tbody>
                    {attributes.map( attr => {
                        return (
                            <tr>
                                <th class="col-md-1" scope="row">{attr.name}</th>
                                <td class="col-md-2">{attr.value}</td>
                            </tr>
                        )
                    })}
                </tbody>
            </bootstrap.Table>
              </bootstrap.Modal.Body>
              <bootstrap.Modal.Footer>
                <bootstrap.Button onClick={this.closeDetails}>Close</bootstrap.Button>
              </bootstrap.Modal.Footer>
            </bootstrap.Modal>
        )
    }
}

class PGNDetails extends React.Component {

    constructor() {
        super();
        this.state = {
            pgn: {id: 0, name: "", edp: 0, dp: 0, pf: 0, ps: "", multipacket: false, dlc: 0},
            spns: [],
            filters:[]
        }
    }

    componentDidMount() {
        fetch("/api/pgn/" + this.props.match.params.pgn_id)
            .then( (response) => {
                return response.json() })   
                    .then( (json) => {
                        fetch("/api/spnforpgn/" + json.id)
                            .then( (response2) => {
                                return response2.json()
                            }).then( (json2) => {
                                fetch("/api/filter/" + json.id)
                                    .then( (response3) => {
                                        return response3.json()
                                    }).then( (json3) => {
                                        this.setState({pgn: json, spns:json2, filters:json3});
                                    })
                            })
                    });
    }

    closeDetails() {
        window.location = "/#/pgn";
    }

    deleteFilter(event) {
        fetch('/api/filter/' + this.state.pgn.id + '/' + event.target.value, {method: "DELETE"})
            .then( success => {
                this.componentDidMount();
        });
    }

    addFilter(event) {
        fetch('/api/filter/' + this.state.pgn.id + '/' + event.target.value, {method: "POST"})
            .then( success => {
                this.componentDidMount();      
        });
    }

    render() {
        let attributes = [
            {name: "ID", value: this.state.pgn.id.toString()},
            {name: "Name", value: this.state.pgn.name},
            {name: "EDP", value: this.state.pgn.edp.toString()},
            {name: "DP", value: this.state.pgn.dp.toString()},
            {name: "PF", value: this.state.pgn.pf.toString()},
            {name: "PS", value: this.state.pgn.ps},
            {name: "Multipacket", value: this.state.pgn.multipacket.toString()},
            {name: "DLC", value: this.state.pgn.dlc.toString()},
        ];
        return (
            <bootstrap.Modal show={true} onHide={this.closeDetails}>
              <bootstrap.Modal.Header>
                <bootstrap.Modal.Title><strong>PGN {this.state.pgn.id}</strong></bootstrap.Modal.Title>
              </bootstrap.Modal.Header>
              <bootstrap.Modal.Body>
            <bootstrap.Table striped bordered condensed hover>
                <thead>
                    <tr>
                        <th scope="col">Attribute</th>
                        <th scope="col">Value</th>
                    </tr>
                </thead>
                <tbody>
                    {attributes.map( attr => {
                        return (
                            <tr>
                                <th class="col-md-1" scope="row">{attr.name}</th>
                                <td class="col-md-2">{attr.value}</td>
                            </tr>
                        )
                    })}
                </tbody>
            </bootstrap.Table>
            <p>Associated with following SPNs</p>
            <bootstrap.Table striped bordered condensed hover>
                <thead>
                    <tr>
                        <th scope="col">ID</th>
                        <th scope="col">Name</th>
                    </tr>
                </thead>
                <tbody>
                    {this.state.spns.map( attr => {
                        let url = "/#/spn/" + attr.id;
                        return (
                            <tr>
                                <th class="col-md-1" scope="row">{attr.id}</th>
                                <td class="col-md-2"><a target="_blank" href={url}>{attr.name}</a></td>
                            </tr>
                        )
                    })}
                </tbody>
            </bootstrap.Table>
            <p>Filters</p>
              <bootstrap.ListGroup>
                {this.state.filters.map (filter => {
                    for(var i = 0; i < filter.for_pgns.length; i++) {
                        if(filter.for_pgns[i].pgn == this.state.pgn.id && filter.for_pgns[i].enabled) {
                            return (
                                <bootstrap.ListGroupItem bsStyle="success" value={filter.id} onClick={this.deleteFilter.bind(this)}>{filter.name}</bootstrap.ListGroupItem>
                            )
                        }
                    }
                    return (
                        <bootstrap.ListGroupItem value={filter.id} onClick={this.addFilter.bind(this)}>{filter.name}</bootstrap.ListGroupItem>
                    )
                })}
              </bootstrap.ListGroup>
              </bootstrap.Modal.Body>
              <bootstrap.Modal.Footer>
                <bootstrap.Button onClick={this.closeDetails}>Close</bootstrap.Button>
              </bootstrap.Modal.Footer>
            </bootstrap.Modal>
        )
    }
}

class SearchableNavigatableTable extends React.Component {
    constructor(props) {
        super(props);
        this.filters = [
            {name: "Contains", func: this.filterContains},
            {name: "Exact Match", func: this.filterExact},
            {name: "Has Prefix", func: this.filterPrefix},
            {name: "Has Suffix", func: this.filterSuffix}
        ];
        this.state = {
            filter: {term: "", filter_func: this.filters[0], target: props.targets[0]}
        };
    }

    filterContains(field, qry) {
        return field.indexOf(qry) !== -1;
    }

    filterExact(field, qry) {
        return field === qry;
    }

    filterPrefix(field, qry) {
        return field.startsWith(qry);
    }

    filterSuffix(field, qry) {
        return field.endsWith(qry);
    }

    resetFilter(event) {
        this.setState({filter: {term: "", filter_func: this.filters[0], target: this.props.targets[0]}});
    }

    updateSearch(event) {
        let updated = this.state.filter;
        updated.term = event.target.value;
        this.updateFilter.bind(this)(updated);
    }

    updateFilter(filter) {
        for(var prop in this.state.filter) {
            if(!filter.hasOwnProperty(prop) && this.state.filter.hasOwnProperty(prop)) {
                filter[prop] = this.state.filter[prop];
            }
        }
        this.setState({filter: filter});
    }

    filterData() {
        if (this.state.filter.target == null) {
            return this.props.data;
        }
        return this.props.data.filter(data => {
            let filter = this.state.filter;
            let field = data.data[filter.target.field];
            if(filter.target.apply != "") {
                if(filter.target.apply_arg != "")
                    field = field[filter.target.apply](filter.target.apply_arg)
                else
                    field = field[filter.target.apply]()
            }
            return filter.filter_func.func(field.toLowerCase(), filter.term.toLowerCase());
        });
    }

    render() {
        return (
            <div id="searchable_navigatable_table">
                <form>
                    <bootstrap.FormGroup>
                        <bootstrap.InputGroup>
                            <bootstrap.InputGroup.Button>
                                <bootstrap.DropdownButton componentClass={bootstrap.InputGroup.Button} bsStyle="primary" title={this.state.filter.target.name}>
                                    {this.props.targets.map( target => {return (<bootstrap.MenuItem onClick={this.updateFilter.bind(this, {target: target})}>{target.name}</bootstrap.MenuItem>)})}
                                </bootstrap.DropdownButton>
                                <bootstrap.DropdownButton componentClass={bootstrap.InputGroup.Button} bsStyle="warning" title={this.state.filter.filter_func.name}>
                                    {this.filters.map(filter => {return (<bootstrap.MenuItem onClick={this.updateFilter.bind(this, {filter_func: filter})}>{filter.name}</bootstrap.MenuItem>)})}
                                </bootstrap.DropdownButton>
                            </bootstrap.InputGroup.Button>
                            <bootstrap.FormControl type="text" placeholder="Filter search..." value={this.state.filter.term} onChange={this.updateSearch.bind(this)} />
                            <bootstrap.InputGroup.Button>
                              <bootstrap.Button bsStyle="danger" >Reset Filter</bootstrap.Button>
                            </bootstrap.InputGroup.Button>
                        </bootstrap.InputGroup>
                    </bootstrap.FormGroup>
                </form>
                <NavigatableTable header={this.props.header} data={this.filterData()}/>
            </div>
        )
    }
}


class NavigatableTable extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            pages: 1,
            page: 1
        };
    }

    changePage(event) {
        let page = parseInt(event.target.value);
        if(page >= 1 && page <= this.state.pages) {
            this.setState({page: page});
        } else {
            this.setState({page: 1});
        }
    }

    createPageNavigation(length) {
        let pages = [];
        this.state.pages = Math.ceil(length / 15);
        let once = true;
        for(var i = 1; i <= this.state.pages; i++) {
            if(i > this.state.page + 4 ) {
                pages.push(<bootstrap.Button disabled>...</bootstrap.Button>);
                pages.push(<bootstrap.Button onClick={this.changePage.bind(this)} value={this.state.pages}>{this.state.pages}</bootstrap.Button>);
                break;
            } else if (i < this.state.page - 4) {
                if (once) {
                    pages.push(<bootstrap.Button onClick={this.changePage.bind(this)} value={1}>{1}</bootstrap.Button>);
                    pages.push(<bootstrap.Button disabled>...</bootstrap.Button>);
                    once = false;
                }
                continue;
            } else if (i === this.state.page) {
                pages.push(<bootstrap.Button onClick={this.changePage.bind(this)} value={i} bsStyle="primary">{i}</bootstrap.Button>);
            } else {
                pages.push(<bootstrap.Button onClick={this.changePage.bind(this)} value={i}>{i}</bootstrap.Button>);
            }
        }
        return pages
    }

    render() {
        return (
            <div id="table_navigation">
                <bootstrap.ButtonToolbar>
                    <bootstrap.ButtonGroup>
                        <bootstrap.Button onClick={this.changePage.bind(this)} value={this.state.page - 1} bsStyle="info">&lt;&lt;</bootstrap.Button>
                        <bootstrap.Button onClick={this.changePage.bind(this)} value={this.state.page + 1} bsStyle="info">&gt;&gt;</bootstrap.Button>
                    </bootstrap.ButtonGroup>
                </bootstrap.ButtonToolbar>
                <bootstrap.ButtonToolbar>
                    <bootstrap.ButtonGroup>
                        <bootstrap.Button disabled>Page</bootstrap.Button>
                        {this.createPageNavigation(this.props.data.length)}
                    </bootstrap.ButtonGroup>
                </bootstrap.ButtonToolbar>
                <Table header={this.props.header} data={this.props.data.slice(0 + (this.state.page - 1) * 15, 15 + (this.state.page - 1) * 15)}/>
            </div>
        )
    }
}

class Table extends React.Component {
    render() {
        return (
            <div id="table">
                <bootstrap.Table striped bordered condensed hover>
                    <thead>
                        <tr>
                            {this.props.header.map( header => {
                                return <th scope="col">{header}</th>
                            })}
                        </tr>
                    </thead>
                    <tbody>
                    {this.props.data.map( row => {
                        let count = 0;
                        if (row.onClickRow == null) {
                            return (
                                <tr>
                                    {row.map( column => {
                                        if (count++ == 0) {
                                            return <th scope="row" class="col-md-1"><div class="clickable">{column}</div></th>
                                        };
                                        return <td class="col-md-2"><div class="clickable long_text">{column}</div></td>
                                    })}
                                </tr>   
                            )
                        } else {
                            return (
                                <tr onClick={row.onClickRow.bind(this, row.paramsOnClick)}>
                                    {row.data.map( column => {
                                        if (count++ == 0) {
                                            return <th scope="row" class="col-md-1"><div class="clickable">{column}</div></th>
                                        };
                                        return <td class="col-md-2"><div class="clickable long_text">{column}</div></td>
                                    })}
                                </tr>   
                            )
                        }
                    })}                    
                  </tbody>
                </bootstrap.Table>
            </div>
        )
    }
}

ReactDOM.render(<App />, document.getElementById('app'));